/*
	chiptuned. squarewave -> arpeggio

	(c) Risto Holopainen 2012
	
	Copying and distribution of this file, with or without modification,
	are permitted in any medium without royalty provided the copyright
	notice and this notice are preserved.  This file is offered as-is,
	without any warranty.
*/


#include "waveset.h"
#include "biquad.h"
#include <cmath>
#include <cstdio>
#include <cstdlib>


double noise()
{
	static double y = 0;
	double x = (double) rand()/RAND_MAX - 0.5;
	
	y = .05*x + .95*y;
	return y;
}



class chip
{
	public:
		chip(int sr);
		~chip();
		double play(float frq);
		double play(float frq, float amp);
		
	private:
		int sr;
		enum {L=512};
		double *wtb;	// square wave
		double *wt2;	// high-pitch wtb
		void make_sq();
		unsigned int t;
		double theta, y, h, x;
		one_pole <double> *LP;
		float cutoff;
		double xi;
		sieve *SV;
};


chip::chip(int fs)
{
	sr = fs;
	t = 0;
	theta = 0.0;
	wtb = new double[L];
	wt2 = new double[L];
	make_sq();
	LP = new one_pole <double> (0.85, .15);
	cutoff = sr/16.;
	SV = new sieve(12,5);
}


chip::~chip()
{
	wtb = 0;
	delete wtb;
	wt2 = 0;
	delete wt2;
	LP = 0;
	delete LP;
}

void chip::make_sq()
{
	double x, W = 2*M_PI/L;
	// puts("wavetable.\n");
	
	for(int i=0; i<L; i++)
		{
		x = 0;
		for(int k=1; k<12; k += 2)
			x += sin(k*W*i) / k;  
		wtb[i] = tanh(x);
		wt2[i] = .85*sin(W*i) + .15*sin(3*W*i);
		//printf("%3d   %.3f\n", i, wtb[i]);
		}
}


double chip::play(float frq)
{
	
	theta += frq/sr;
	while(theta >= 1.0)
		theta -= 1.0;
	while(theta < 0.0)
		theta += 1.0;

	t = (unsigned int) (theta * L);
	if(frq < sr/12)
		y = wtb[t];
		
	else if(frq >= sr/12.)
		{
		h = (frq - sr/12.) / (sr/3. - sr/12.);
		if(frq > sr/3.) 
			h = 1;
		y = h*wt2[t] + (1-h)*wtb[t];
		}

	return LP->filter(y);
}




// add breathing--phrasing
double chip::play(float frq, float a)
{
	xi = noise();
	theta += (frq + (1-a)*xi)/sr;

	while(theta >= 1.0)
		theta -= 1.0;
	while(theta < 0.0)
		theta += 1.0;

	t = (unsigned int) (theta * L);
	if(frq < cutoff)
		y = wtb[t];
		
	else if(frq >= cutoff)
		{
		h = (frq - cutoff) / (sr/3. - cutoff);
		if(frq > sr/3.) 
			h = 1;
		y = h*wt2[t] + (1-h)*wtb[t];
		}



	x = SV->operator()(y + xi);
	a = sqrt(fabs(a));
	float G = (frq > 500) ? 500. / frq : 1;
	return LP->filter( G*a*y + (1-a)*x );

}




